﻿'**************************************************************
' VB .NET PDF417 Class Font Encoder for IDAutomation Barcode 
' Fonts V.2020
' © Copyright, 2020 IDAutomation.com, Inc. All rights reserved.
' This function returns a text string as an Expression. This
' string will generate the barcode when combined with the
' correct barcode font. 
' Font to use: IDAutomation2D 
'
''*****************************************************************
'' Limitations of the Demo Version:
'' The demo version of this product includes a fully
'' functional 2D font and is designed only for the
'' purpose of testing the font in various applications
'' and point sizes. To generate dynamic symbols,
'' a font encoder is required to format the data being
'' encoded into the 2D symbol.
'' Only static data can be provided for the encoders
'' in the demo version, because providing a working
'' encoder would reveal the complete source code,
'' which is only available in the licensed version.
'' 
''*****************************************************************
''
'' Please refer to the User Manual and Included Examples
'' https://www.idautomation.com/barcode-components/dotnet-standard/
''
''*****************************************************************

Imports System.Text


Public Class PDF417

    Private Matrix(,) As Integer
    Private rows As Integer = 0
    Private cols As Integer = 0
    Private _newline As String = ""
    Private ResultRows() As String

    '' <summary>
    '' Function, which creates a PDF417 barcode, depending on the various input options chosen. 
    '' </summary>
    '' <param name="DataToEncode">The data that Is encoded in the barcode.</param>
    '' <param name="ApplyTilde">Enables processing of tilde And parentheses characters for purposes of encoding ASCII functions And GS1 application identifiers.</param>
    '' <param name="EccLevel">The level of error correction used to create the barcode.</param>
    '' <param name="EncodingMode">Various modes used to Data represented in the symbol may be encoded by one of these types.</param>
    '' <param name="NumColumns">The number of columns in the PDF417 barcode.</param>
    '' <param name="NumRows">The number of rows in the PDF417 barcode.</param>
    '' <param name="Truncate">The right hand side of the PDF417 Is removed Or truncated.</param>
    '' <param name="QuietZone">Define the size of a blank zone around the code.</param>
    '' <param name="ModuleSize">Define the size of each module.</param>
    '' <returns>A string representation of vector PDF417's image.</returns>
    Public Function EncodePDF417(DataToEncode As String, Optional ApplyTilde As Boolean = True, Optional EccLevel As Integer = 0, Optional EncodingMode As EncodingModes = EncodingModes.Binary,
                                 Optional NumColumns As Integer = 0, Optional NumRows As Integer = 0, Optional Truncate As Boolean = False, Optional QuietZone As Integer = 2, Optional ModuleSize As Integer = 1)


        rows = 24
        cols = 121

        Dim Matrix(,) As Integer = {
        {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0}
            }

        Dim Result As Byte()
        Dim k As Integer
        Dim intPad As Integer = 0
        Dim pixels As Integer
        Dim bit As Integer
        Dim bWidth As Integer
        Dim x As Integer
        Dim y As Integer
        Dim m As Integer
        Dim l As Integer

        bWidth = (Math.Floor((cols * ModuleSize) / 8) + (If((cols * ModuleSize) Mod 8 > 0, 1, 0)))
        Dim intTemp As Integer = bWidth Mod 4
        If (intTemp <> 0) Then
            intPad = 4 - intTemp
        End If

        ReDim Result(64 + rows * ModuleSize * (bWidth + intPad) - 1)


        Result(0) = &H42
        Result(1) = &H4D
        For k = 0 To 7
            Result(2 + k) = 0
        Next
        Result(10) = &H3E
        For k = 0 To 2
            Result(11 + k) = 0
        Next
        Result(14) = &H28
        For k = 0 To 2
            Result(15 + k) = 0
        Next

        '' width
        Result(18) = CByte(Math.Floor((cols * ModuleSize) Mod 256))
        Result(19) = CByte(Math.Floor((cols * ModuleSize) / 256))
        Result(20) = 0
        Result(21) = 0

        '' height
        Result(22) = CByte(Math.Floor((rows * ModuleSize) Mod 256))
        Result(23) = CByte(Math.Floor((rows * ModuleSize) / 256))
        Result(24) = 0
        Result(25) = 0

        Result(26) = 1
        Result(27) = 0
        Result(28) = 1
        Result(29) = 0
        For k = 0 To 7
            Result(30 + k) = 0
        Next
        Result(38) = &H25
        Result(39) = &HE
        Result(40) = 0
        Result(41) = 0

        Result(42) = &H25
        Result(43) = &HE
        Result(44) = 0
        Result(45) = 0

        For k = 0 To 7
            Result(46 + k) = 0
        Next

        For k = 0 To 2
            Result(54 + k) = &HFF
        Next
        For k = 0 To 4
            Result(57 + k) = 0
        Next

        Dim pos As Integer = 62

        For y = rows - 1 To 0 Step -1

            For m = 0 To ModuleSize - 1

                pixels = 0
                bit = 8

                For x = 0 To cols - 1
                    If ((Matrix(y, x) <> 0)) Then
                        For l = 0 To ModuleSize - 1
                            bit = bit - 1
                            pixels = pixels + Math.Floor(Math.Pow(2, bit))
                            If (bit = 0) Then
                                Result(pos) = CByte(pixels)
                                pos += 1
                                bit = 8
                                pixels = 0
                            End If
                        Next

                    Else
                        For l = 0 To ModuleSize - 1
                            bit = bit - 1
                            If (bit = 0) Then
                                Result(pos) = CByte(pixels)
                                pos += 1
                                bit = 8
                                pixels = 0
                            End If
                        Next

                    End If
                Next
                If (bit <> 8) Then
                    Result(pos) = CByte(pixels)
                    pos += 1
                End If
                pos = pos + intPad
            Next
        Next

        Return Result

    End Function



    '' <summary>
    '' Function, which creates a PDF417 barcode, depending on the various input options chosen. 
    '' </summary>
    '' <param name="DataToEncode">The data that Is encoded in the barcode.</param>
    '' <param name="ApplyTilde">Enables processing of tilde And parentheses characters for purposes of encoding ASCII functions And GS1 application identifiers.</param>
    '' <param name="EccLevel">The level of error correction used to create the barcode.</param>
    '' <param name="EncodingMode">Various modes used to Data represented in the symbol may be encoded by one of these types.</param>
    '' <param name="NumColumns">The number of columns in the PDF417 barcode.</param>
    '' <param name="NumRows">The number of rows in the PDF417 barcode.</param>
    '' <param name="Truncate">The right hand side of the PDF417 Is removed Or truncated.</param>
    '' <param name="VectorType">Define the type of vector image for output.</param>
    '' <param name="NewLine">String indicating EOL character. Defaults to the operating system EOL character if left NULL.</param>
    '' <param name="QuietZone">Define the size of a blank zone around the code.</param>
    '' <param name="ModuleSize">Define the size of each module.</param>
    '' <returns>A string representation of vector PDF417's image.</returns>
    Public Function EncodePDF417(DataToEncode As String, Optional ApplyTilde As Boolean = True, Optional EccLevel As Integer = 0, Optional EncodingMode As EncodingModes = EncodingModes.Binary,
                                 Optional NumColumns As Integer = 0, Optional NumRows As Integer = 0, Optional Truncate As Boolean = False, Optional VectorType As VectorTypes = VectorTypes.VecEPS, Optional NewLine As String = "", Optional QuietZone As Integer = 2, Optional ModuleSize As Integer = 2)



        rows = 24
        cols = 121

        Dim Matrix(,) As Integer = {
        {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0},
             {1, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 1, 0, 0, 1, 0}
            }

        '' allows user to set an end-of-line (EOL) string
        If (NewLine = "") Then
            NewLine = Environment.NewLine '' Default
        End If
        _newline = NewLine

        Dim strOut As StringBuilder = New StringBuilder("")
        Dim tempW As String
        Dim tempH As String
        Dim x As Integer
        Dim y As Integer

        ModuleSize = ModuleSize * 4
        tempW = (cols * ModuleSize).ToString()
        tempH = (rows * ModuleSize).ToString()

        If (VectorType = VectorTypes.VecSVG) Then

            strOut.Append("<?xml version='1.0' standalone='no'?>" + _newline)
            strOut.Append("<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 1.1//EN' 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd'>" + _newline)
            strOut.Append("<svg width='" + tempW + "' height='" + tempH + "' version='1.1' xmlns='http://www.w3.org/2000/svg'>" + _newline)
            strOut.Append("  <g id='PDF417'>" + _newline)
            strOut.Append("    <g id='background' fill='#ffffff' stroke='#ffffff'>" + _newline)
            strOut.Append("      <rect x='0' y='0' width='" + tempW + "' height='" + tempH + "' />" + _newline)
            strOut.Append("    </g>" + _newline)
            strOut.Append("    <g id='pattern' fill='#000000' stroke='#000000'>" + _newline)

            For y = 0 To rows - 1
                For x = 0 To cols - 1
                    If (Matrix(y, x) <> 0) Then
                        strOut.Append("    <g id='R" + y.ToString() + "C" + x.ToString() + "' >" + _newline)
                        strOut.Append("      <rect x='" + (x * ModuleSize).ToString() + "' y='" + (y * ModuleSize).ToString() + "' width='" + (ModuleSize).ToString() + "' height='" + (ModuleSize).ToString() + "' />" + _newline)
                        strOut.Append("    </g>" + _newline)
                    End If
                Next
            Next
            strOut.Append("    </g>" + _newline)
            strOut.Append("  </g>" + _newline)
            strOut.Append("</svg>" + _newline)

        ElseIf (VectorType = VectorTypes.VecEPS) Then
            strOut.Append("%!PS-Adobe-3.0 EPSF-3.0" + _newline)
            strOut.Append("%%Creator: IDAutomation.com" + _newline)
            strOut.Append("%%Title: pdf417" + _newline)
            strOut.Append("%%CreationDate: " + DateTime.Now.ToString("ddd mmm d hh:mm:ss yyyy") + _newline)
            strOut.Append("%%BoundingBox: 0 0 " + tempW + " " + tempH + _newline)
            strOut.Append("%%DocumentData: Clean7Bit" + _newline)
            strOut.Append("%%EndComments" + _newline)
            strOut.Append("%%BeginProlog" + _newline)
            strOut.Append("/bd { bind def } bind def" + _newline)
            strOut.Append("/incompound false def" + _newline)
            strOut.Append("/m { moveto } bd" + _newline)
            strOut.Append("/l { lineto } bd" + _newline)
            strOut.Append("/*U { /incompound false def f} bd" + _newline)
            strOut.Append("/k { setcmykcolor } bd" + _newline)

            strOut.Append("/c { curveto } bd" + _newline)
            strOut.Append("/F { incompound not {fill} if } bd" + _newline)
            strOut.Append("/f { closepath F } bd" + _newline)
            strOut.Append("/S { stroke } bd" + _newline)
            strOut.Append("/*u { /incompound true def } bd" + _newline)
            strOut.Append("/K { k } bd" + _newline)

            strOut.Append("%%EndProlog" + _newline)
            strOut.Append("%%BeginSetup" + _newline)
            strOut.Append("%%EndSetup" + _newline)
            strOut.Append("0.000 0.000 0.000 0.000 k" + _newline)
            strOut.Append("0 0 m" + _newline)
            strOut.Append("0 0 l" + _newline)
            strOut.Append(tempW + " 0 l" + _newline)
            strOut.Append(tempW + " " + tempH + " l" + _newline)
            strOut.Append("0 " + tempH + " l" + _newline)
            strOut.Append("f" + _newline)
            strOut.Append("*U" + _newline)

            Dim n As Integer = rows * ModuleSize
            strOut.Append("0.000 0.000 0.000 1.000 k" + _newline)
            For y = 0 To rows - 1
                For x = 0 To cols - 1
                    If (Matrix(y, x) <> 0) Then
                        strOut.Append((x * ModuleSize).ToString() + " " + (n - y * ModuleSize).ToString() + " m" + _newline)
                        strOut.Append((x * ModuleSize).ToString() + " " + (n - y * ModuleSize).ToString() + " l" + _newline)
                        strOut.Append(((x + 1) * ModuleSize).ToString() + " " + (n - y * ModuleSize).ToString() + " l" + _newline)
                        strOut.Append(((x + 1) * ModuleSize).ToString() + " " + (n - (y + 1) * ModuleSize).ToString() + " l" + _newline)
                        strOut.Append(((x) * ModuleSize).ToString() + " " + (n - (y + 1) * ModuleSize).ToString() + " l" + _newline)
                        strOut.Append("f" + _newline)
                    End If
                Next
            Next

            strOut.Append("*U" + _newline)
            strOut.Append("%%Trailer" + _newline)
            strOut.Append("%%EOF" + _newline)

        ElseIf (VectorType = VectorTypes.VecHTML) Then
            strOut.Append("<div style='font-size:0;background-color:#ffffff;position:relative;width:" + tempW + "px;height:" + tempH + "px;'>" + _newline)
            For y = 0 To rows - 1
                For x = 0 To cols - 1
                    If (Matrix(y, x) <> 0) Then
                        strOut.Append("<div style='background-color:#000000;width:" + ModuleSize.ToString() + "px;height:" + ModuleSize.ToString() + "px;position:absolute;left:" + (x * ModuleSize).ToString() + "px;top:" + (y * ModuleSize).ToString() + "px;'>&nbsp</div>" + _newline)
                    End If
                Next
            Next

            strOut.Append("</div>" + _newline)
        End If

        Return strOut.ToString()

    End Function

    '' <summary>
    '' This function encodes the data passed in into a suitable format so that the barcode font can be applied to the data.
    '' </summary>
    '' <param name="DataToEncode">The data to be encoded in the barcode.</param>
    '' <param name="ApplyTilde">Determines if data after the tilde character should be interpreted as an ASCII code.</param>
    '' <param name="EccLevel">The level of error correction used to create the barcode.</param>
    '' <param name="EncodingMode">Various modes used to Data represented in the symbol may be encoded by one of these types.</param>
    '' <param name="NumColumns">The number of columns in the PDF417 barcode.</param>
    '' <param name="NumRows">The number of rows in the PDF417 barcode.</param>
    '' <param name="Truncate">The right hand side of the PDF417 Is removed Or truncated.</param>
    '' <param name="OutputType">Define the type of font for output.</param>
    '' <param name="NewLine">String indicating EOL character. Defaults to the operating system EOL character if left NULL.</param>
    '' <returns>Encoded text for use with the 2D barcode font.</returns>
    Public Function EncodePDF417(DataToEncode As String, Optional ApplyTilde As Boolean = True, Optional EccLevel As Integer = 0, Optional EncodingMode As EncodingModes = EncodingModes.Binary,
                                 Optional NumColumns As Integer = 0, Optional NumRows As Integer = 0, Optional Truncate As Boolean = False, Optional OutputType As OutputTypes = OutputTypes.IDA2DFont, Optional NewLine As String = "") As String


        '' allows user to set an end-of-line (EOL) string
        If (NewLine = "") Then
            NewLine = Environment.NewLine '' Default
        End If
        _newline = NewLine

        If (OutputType = OutputTypes.IDA2DFont) Then

            EncodePDF417 = "777777770707070007676724353544662075407375124766230760111660572235707316553674241111077765252407555510777777707000707007" + NewLine + "777777770707070007735347671154774071444131776212050732743755552647407422377607523114077361657731476640777777707000707007" + NewLine
        Else
            EncodePDF417 = "████████ █ █ █   █████ █ █ █████  ███ █ ██  ████   ██    ██ ██   ██ █  ███ ███ █     █████ █ █ █████  ███████ █   █ █  █" + NewLine + "████████ █ █ █   █████▄▀▄▀▄▀▀▀██▄ █▀▀ █▄█▀ ▄▀███▄▄ ██    ██ ▀█▄▄▄▀█ █▄ █▀▀▄██▀▄▀     ████▀▄▀▄▀ █▀▀▀▀  ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ██████ █ █   ███ █   ███  █ █████ ██    ██  ████ █ ██ █  ███ █      ████ █ █  █      ███████ █   █ █  █ " + NewLine + "████████ █ █ █   █ █ █  ████      ██  █████  █   █ █  ███   ██  ███ ███ ███ █   ████ ███ █ █   ██████ ███████ █   █ █  █ " + NewLine + "████████ █ █ █   █▄▀▄▀▄▄██▀▀▄▄▄▄▄ █▀▄▄█▀▀▀█▄▄▀   █ █  ██▀▄▄▄██ ▄███ ██▀ ▀██▄▀▄▄ ▀▀▀█ ██▀▄▀▄█▄▄ ▀█████ ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ██ █ ████  █████ █ ███   ███    █ █  ██ █████ ████ ██   ███ ██    █ ██ █ ████  █████ ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ███ █ ███    ██  █     █ ████ █   ████ ██    ██ █  █ ██████ █ ██    ████ █ ███  ███  ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ███▄█ █▀█▄▄▄ ██  █▄   ▄█▄██▀▀▄▀ ▄ ██▀█ ██▄▄▄▄▀▀ █  █ ▀▀███▀ █▄▀█▄▄  ███▀▄▀▄███▄ █▀▀  ███████ █   █ █  █ " + NewLine + "████████ █ █ █   █████ █ ████ ██  ██   █████  █  █ ██ █ ██████   █  █   ███  ██ ███  ███ █ █████ █    ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ███ █  ███ █████ ███ ████ ██   █  ██ ██     █    █ ████ ██ █    ███ ███ █  ███ █████ ███████ █   █ █  █ " + NewLine + "████████ █ █ █   ███▄█ ▄▀▀█ ██▀▀▀ █▀▀▄██▀▀▄▀█▄▄▄█  ██ ▀▀ ▄ ▄▄█   ▄▀ ██▀▀ ▀█ █▄▄  ▀█▀ █▀█ █▄▄███ ███▀▀ ███████ █   █ █  █ " + NewLine + "████████ █ █ █   █████ █  █ ██    █  ███  █ █████  ██    █ ███   █  ██    █ ███   █  █ █ ██████ ███   ███████ █   █ █  █ " + NewLine + ""

        End If

    End Function


End Class


'' <summary>
'' Various modes used to Data represented in the symbol may be encoded by one of these types.
'' </summary>
Public Enum EncodingModes
    '' <summary>
    '' In this mode data Is encoded at 8 bits per character.
    '' </summary>
    Binary

    '' <summary>
    '' Use a TEXT encoding chartset optimized for data that only contains alphanumeric characters. 
    '' </summary>
    Text

End Enum

'' <summary>
'' Various types of output.
'' </summary>
Public Enum OutputTypes

    '' <summary>
    '' For use with IDAutomation PDF417 Or 2D Barcode Fonts. Returns PDF417 encoded text. 
    '' </summary>
    IDA2DFont


#If Not ENCODER Then
    '' <summary>
    '' Returns PDF417 native object. For use with monospaced fonts.
    '' </summary>
    IDAMonospaceFont

    '' <summary>
    '' Returns PDF417 native object. For use with standard (proportional) fonts.
    '' </summary>

    IDAProportionalFont
#End If

End Enum


'' <summary>
'' Various types of vector image output.
'' </summary>
Public Enum VectorTypes

    '' <summary>
    '' Returns a string with the SVG representation. 
    '' </summary>
    VecSVG

    '' <summary>
    '' Return a string with the EPS representation.
    '' </summary>
    VecEPS

    '' <summary>
    '' Return a string with the HTML representation.
    '' </summary>
    VecHTML

End Enum