﻿using System;
using Xamarin.Forms;
using Xamarin.Forms.Xaml;
using IDAutomation.NetStandard.PDF417.FontEncoder;
using System.Collections.Generic;
using System.IO;

namespace XamarinTest
{
    [XamlCompilation(XamlCompilationOptions.Compile)]
    public partial class PDF417Page : ContentPage

         
    {

        PDF417 pdf417 = new PDF417(); //local instance
        Label lblcode = new Label { };
        Editor editor = new Editor { };
        Image image = new Image { };

        // Dictionary for font picker.
        Dictionary<string, int> outputMode = new Dictionary<string, int>
        {
            { "IDAutomation2D font", 0 }, { "Unicode Image (Courier New)", 1 }, { "BMP Image ", 2 }
        };

        Picker picker = new Picker
        {
            Title = "Output mode",
            VerticalOptions = LayoutOptions.CenterAndExpand
        };



        public PDF417Page()
        {

            var layout = new StackLayout { Padding = new Thickness(5, 5) };
            this.Content = layout;
            editor.Text = "IDAutomation .Net Standard 2";
            layout.Children.Add(editor);

            foreach (string outputType in outputMode.Keys)
            {
                picker.Items.Add(outputType);
            }
            picker.SelectedIndex = 0;
            layout.Children.Add(picker);


            lblcode.FontFamily = Device.RuntimePlatform == Device.iOS ? "IDAutomation2D" :
                                   Device.RuntimePlatform == Device.Android ? "IDAutomation2D.ttf#IDAutomation2D" : "Assets/IDAutomation2D.ttf#IDAutomation2D";
            lblcode.TextColor = Color.Black;
            lblcode.VerticalOptions = LayoutOptions.Center;
            lblcode.HorizontalOptions = LayoutOptions.Center;
            lblcode.FontSize = Device.RuntimePlatform == Device.iOS ? 18 :
                Device.RuntimePlatform == Device.Android ? Device.GetNamedSize(NamedSize.Medium, lblcode) : Device.GetNamedSize(NamedSize.Large, lblcode);
            layout.Children.Add(lblcode);

            image.BackgroundColor = Color.White;
            image.VerticalOptions = LayoutOptions.Center;
            image.HorizontalOptions = LayoutOptions.Center;
            image.IsVisible = false;
            layout.Children.Add(image);



            Button button = new Button
            {
                Text = "Apply",
                VerticalOptions = LayoutOptions.CenterAndExpand,
                HorizontalOptions = LayoutOptions.Center
            };
            button.Clicked += apply;
            layout.Children.Add(button);

        }

        void apply(object sender, EventArgs e)
        {
            if (picker.SelectedIndex == 0)
            {
                lblcode.IsVisible = true;
                image.IsVisible = false;

                lblcode.FontFamily = Device.RuntimePlatform == Device.iOS ? "IDAutomation2D" :
                                   Device.RuntimePlatform == Device.Android ? "IDAutomation2D.ttf#IDAutomation2D" : "Assets/IDAutomation2D.ttf#IDAutomation2D";
                lblcode.FontSize = Device.RuntimePlatform == Device.iOS ? 18 :
                    Device.RuntimePlatform == Device.Android ? Device.GetNamedSize(NamedSize.Default, lblcode) : Device.GetNamedSize(NamedSize.Medium, lblcode);
                lblcode.Text = pdf417.EncodePDF417(editor.Text, true, 0, EncodingModes.Binary, 0, 0, false, OutputTypes.IDA2DFont);
            }
            else if (picker.SelectedIndex == 1)
            {
                lblcode.IsVisible = true;
                image.IsVisible = false;
                lblcode.FontFamily = Device.RuntimePlatform == Device.iOS ? "Courier New" :
                                   Device.RuntimePlatform == Device.Android ? "Courier.ttf#Courier New" : "Assets/Courier.ttf#Courier New";
                lblcode.FontSize = 5; // Device.RuntimePlatform == Device.iOS ? 10 :
                                   //Device.RuntimePlatform == Device.Android ? Device.GetNamedSize(NamedSize.Small, lblcode) : Device.GetNamedSize(NamedSize.Default, lblcode);
                lblcode.Text = pdf417.EncodePDF417(editor.Text, true, 0, EncodingModes.Binary, 0, 0, false, OutputTypes.IDAMonospaceFont);
            }
            else

            {
                image.IsVisible = true;
                lblcode.IsVisible = false;
                image.Source = ImageSource.FromStream(() => new MemoryStream(pdf417.EncodePDF417(editor.Text, true, 0, EncodingModes.Binary,0,0,false, 2, 10)));
            }


        }
    }
}

